from __future__ import annotations

from dataclasses import dataclass, field

from momento_wire_types import vectorindex_pb2 as pb

from momento.errors.exceptions import InvalidArgumentException
from momento.internal.services import Service


# TODO: support other datatypes for the vector (np.array, pd.Series, etc.)
@dataclass
class Item:
    """Represents an entry in the vector index."""

    id: str
    """The id of the item."""

    vector: list[float]
    """The vector of the item."""

    metadata: dict[str, str] = field(default_factory=dict)
    """The metadata of the item."""

    def to_proto(self) -> pb._Item:
        vector = pb._Vector(elements=self.vector)
        metadata = []
        if self.metadata is not None:
            for k, v in self.metadata.items():
                if type(v) is not str:
                    raise InvalidArgumentException(
                        f"Metadata values must be strings. Field {k!r} has a value of type {type(v)!r} with value {v!r}.",  # noqa: E501
                        Service.INDEX,
                    )
                metadata.append(pb._Metadata(field=k, string_value=v))

        return pb._Item(id=self.id, vector=vector, metadata=metadata)



from __future__ import annotations

from abc import ABC
from dataclasses import dataclass, field

from momento_wire_types import vectorindex_pb2 as vectorindex_pb

from ...mixins import ErrorResponseMixin
from ..response import VectorIndexResponse


class SearchResponse(VectorIndexResponse):
    """Parent response type for a vector index `search` request.

    Its subtypes are:
    - `Search.Success`
    - `Search.Error`

    See `VectorIndexClient` for how to work with responses.
    """


@dataclass
class SearchHit:
    id: str
    distance: float
    metadata: dict[str, str] = field(default_factory=dict)

    @staticmethod
    def from_proto(hit: vectorindex_pb._SearchHit) -> SearchHit:
        metadata = {  # type: ignore
            item.field: item.string_value  # type: ignore
            for item in hit.metadata  # type: ignore
        }
        return SearchHit(id=hit.id, distance=hit.distance, metadata=metadata)  # type: ignore


class Search(ABC):
    """Groups all `SearchResponse` derived types under a common namespace."""

    @dataclass
    class Success(SearchResponse):
        """Indicates the request was successful."""

        hits: list[SearchHit]

    class Error(SearchResponse, ErrorResponseMixin):
        """Contains information about an error returned from a request.

        This includes:
        - `error_code`: `MomentoErrorCode` value for the error.
        - `messsage`: a detailed error message.
        """



from .control.create import CreateIndex, CreateIndexResponse
from .control.delete import DeleteIndex, DeleteIndexResponse
from .control.list import ListIndexes, ListIndexesResponse
from .data.delete_item_batch import DeleteItemBatch, DeleteItemBatchResponse
from .data.search import Search, SearchHit, SearchResponse
from .data.upsert_item_batch import UpsertItemBatch, UpsertItemBatchResponse
from .response import VectorIndexResponse

__all__ = [
    "CreateIndex",
    "CreateIndexResponse",
    "DeleteIndex",
    "DeleteIndexResponse",
    "ListIndexes",
    "ListIndexesResponse",
    "SearchHit",
    "Search",
    "SearchResponse",
    "UpsertItemBatch",
    "UpsertItemBatchResponse",
    "DeleteItemBatch",
    "DeleteItemBatchResponse",
    "VectorIndexResponse",
]
